=== added file 'bzrlib/cleanup.py'
--- bzrlib/cleanup.py	1970-01-01 00:00:00 +0000
+++ bzrlib/cleanup.py	2009-10-15 02:19:43 +0000
@@ -0,0 +1,154 @@
+# Copyright (C) 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Helpers for managing cleanup functions and the errors they might raise.
+
+The usual way to run cleanup code in Python is::
+
+    try:
+        do_something()
+    finally:
+        cleanup_something()
+
+However if both `do_something` and `cleanup_something` raise an exception
+Python will forget the original exception and propagate the one from
+cleanup_something.  Unfortunately, this is almost always much less useful than
+the original exception.
+
+If you want to be certain that the first, and only the first, error is raised,
+then use::
+
+    do_with_cleanups(do_something, cleanups)
+
+This is more inconvenient (because you need to make every try block a
+function), but will ensure that the first error encountered is the one raised,
+while also ensuring all cleanups are run.
+"""
+
+
+import sys
+from bzrlib import (
+    debug,
+    trace,
+    )
+
+def _log_cleanup_error(exc):
+    trace.mutter('Cleanup failed:')
+    trace.log_exception_quietly()
+    if 'cleanup' in debug.debug_flags:
+        trace.warning('bzr: warning: Cleanup failed: %s', exc)
+
+
+def _run_cleanup(func, *args, **kwargs):
+    """Run func(*args, **kwargs), logging but not propagating any error it
+    raises.
+
+    :returns: True if func raised no errors, else False.
+    """
+    try:
+        func(*args, **kwargs)
+    except KeyboardInterrupt:
+        raise
+    except Exception, exc:
+        _log_cleanup_error(exc)
+        return False
+    return True
+
+
+def _run_cleanup_reporting_errors(func, *args, **kwargs):
+    try:
+        func(*args, **kwargs)
+    except KeyboardInterrupt:
+        raise
+    except Exception, exc:
+        trace.mutter('Cleanup failed:')
+        trace.log_exception_quietly()
+        trace.warning('Cleanup failed: %s', exc)
+        return False
+    return True
+
+
+def _run_cleanups(funcs, on_error='log'):
+    """Run a series of cleanup functions.
+
+    :param errors: One of 'log', 'warn first', 'warn all'
+    """
+    seen_error = False
+    for func in funcs:
+        if on_error == 'log' or (on_error == 'warn first' and seen_error):
+            seen_error |= _run_cleanup(func)
+        else:
+            seen_error |= _run_cleanup_reporting_errors(func)
+
+
+def do_with_cleanups(func, cleanup_funcs):
+    """Run `func`, then call all the cleanup_funcs.
+
+    All the cleanup_funcs are guaranteed to be run.  The first exception raised
+    by func or any of the cleanup_funcs is the one that will be propagted by
+    this function (subsequent errors are caught and logged).
+
+    Conceptually similar to::
+
+        try:
+            return func()
+        finally:
+            for cleanup in cleanup_funcs:
+                cleanup()
+
+    It avoids several problems with using try/finally directly:
+     * an exception from func will not be obscured by a subsequent exception
+       from a cleanup.
+     * an exception from a cleanup will not prevent other cleanups from
+       running (but the first exception encountered is still the one
+       propagated).
+
+    Unike `run_cleanup`, `do_with_cleanups` can propagate an exception from a
+    cleanup, but only if there is no exception from func.
+    """
+    # As correct as Python 2.4 allows.
+    try:
+        result = func()
+    except:
+        # We have an exception from func already, so suppress cleanup errors.
+        _run_cleanups(cleanup_funcs)
+        raise
+    else:
+        # No exception from func, so allow the first exception from
+        # cleanup_funcs to propagate if one occurs (but only after running all
+        # of them).
+        exc_info = None
+        for cleanup in cleanup_funcs:
+            # XXX: Hmm, if KeyboardInterrupt arrives at exactly this line, we
+            # won't run all cleanups... perhaps we should temporarily install a
+            # SIGINT handler?
+            if exc_info is None:
+                try:
+                    cleanup()
+                except:
+                    # This is the first cleanup to fail, so remember its
+                    # details.
+                    exc_info = sys.exc_info()
+            else:
+                # We already have an exception to propagate, so log any errors
+                # but don't propagate them.
+                _run_cleanup(cleanup)
+        if exc_info is not None:
+            raise exc_info[0], exc_info[1], exc_info[2]
+        # No error, so we can return the result
+        return result
+
+

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2009-10-08 22:53:13 +0000
+++ bzrlib/repository.py	2009-10-15 02:11:18 +0000
@@ -50,7 +50,6 @@
 """)
 
 from bzrlib.decorators import needs_read_lock, needs_write_lock, only_raises
-from bzrlib.lock import _RelockDebugMixin
 from bzrlib.inter import InterObject
 from bzrlib.inventory import (
     Inventory,
@@ -58,6 +57,7 @@
     ROOT_ID,
     entry_factory,
     )
+from bzrlib.lock import _RelockDebugMixin
 from bzrlib import registry
 from bzrlib.trace import (
     log_exception_quietly, note, mutter, mutter_callsite, warning)

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2009-10-13 08:46:44 +0000
+++ bzrlib/tests/__init__.py	2009-10-15 02:11:18 +0000
@@ -3720,6 +3720,7 @@
         'bzrlib.tests.test_chk_serializer',
         'bzrlib.tests.test_chunk_writer',
         'bzrlib.tests.test_clean_tree',
+        'bzrlib.tests.test_cleanup',
         'bzrlib.tests.test_commands',
         'bzrlib.tests.test_commit',
         'bzrlib.tests.test_commit_merge',

=== added file 'bzrlib/tests/test_cleanup.py'
--- bzrlib/tests/test_cleanup.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_cleanup.py	2009-10-15 02:19:43 +0000
@@ -0,0 +1,259 @@
+# Copyright (C) 2009 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+from cStringIO import StringIO
+import re
+
+from bzrlib.cleanup import (
+    do_with_cleanups,
+    _run_cleanup,
+    )
+from bzrlib.tests import TestCase
+from bzrlib import (
+    debug,
+    trace,
+    )
+
+
+class CleanupsTestCase(TestCase):
+
+    def setUp(self):
+        super(CleanupsTestCase, self).setUp()
+        self.call_log = []
+
+    def no_op_cleanup(self):
+        self.call_log.append('no_op_cleanup')
+
+    def assertLogContains(self, regex):
+        log = self._get_log(keep_log_file=True)
+        self.assertContainsRe(log, regex, re.DOTALL)
+
+    def failing_cleanup(self):
+        self.call_log.append('failing_cleanup')
+        raise Exception("failing_cleanup goes boom!")
+
+
+class TestRunCleanup(CleanupsTestCase):
+
+    def test_no_errors(self):
+        """The function passed to _run_cleanup is run."""
+        self.assertTrue(_run_cleanup(self.no_op_cleanup))
+        self.assertEqual(['no_op_cleanup'], self.call_log)
+
+    def test_cleanup_with_args_kwargs(self):
+        def func_taking_args_kwargs(*args, **kwargs):
+            self.call_log.append(('func', args, kwargs))
+        _run_cleanup(func_taking_args_kwargs, 'an arg', kwarg='foo')
+        self.assertEqual(
+            [('func', ('an arg',), {'kwarg': 'foo'})], self.call_log)
+
+    def test_cleanup_error(self):
+        """An error from the cleanup function is logged by _run_cleanup, but not
+        propagated.
+
+        This is there's no way for _run_cleanup to know if there's an existing
+        exception in this situation::
+            try:
+              some_func()
+            finally:
+              _run_cleanup(cleanup_func)
+        So, the best _run_cleanup can do is always log errors but never raise
+        them.
+        """
+        self.assertFalse(_run_cleanup(self.failing_cleanup))
+        self.assertLogContains('Cleanup failed:.*failing_cleanup goes boom')
+
+    def test_cleanup_error_debug_flag(self):
+        """The -Dcleanup debug flag causes cleanup errors to be reported to the
+        user.
+        """
+        log = StringIO()
+        trace.push_log_file(log)
+        debug.debug_flags.add('cleanup')
+        self.assertFalse(_run_cleanup(self.failing_cleanup))
+        self.assertContainsRe(
+            log.getvalue(),
+            "bzr: warning: Cleanup failed:.*failing_cleanup goes boom")
+
+    def test_prior_error_cleanup_succeeds(self):
+        """Calling _run_cleanup from a finally block will not interfere with an
+        exception from the try block.
+        """
+        def failing_operation():
+            try:
+                1/0
+            finally:
+                _run_cleanup(self.no_op_cleanup)
+        self.assertRaises(ZeroDivisionError, failing_operation)
+        self.assertEqual(['no_op_cleanup'], self.call_log)
+
+    def test_prior_error_cleanup_fails(self):
+        """Calling _run_cleanup from a finally block will not interfere with an
+        exception from the try block even when the cleanup itself raises an
+        exception.
+
+        The cleanup exception will be logged.
+        """
+        def failing_operation():
+            try:
+                1/0
+            finally:
+                _run_cleanup(self.failing_cleanup)
+        self.assertRaises(ZeroDivisionError, failing_operation)
+        self.assertLogContains('Cleanup failed:.*failing_cleanup goes boom')
+
+
+#class TestRunCleanupReportingErrors(CleanupsTestCase):
+#
+#    def test_cleanup_error_reported(self):
+#        xxx
+
+
+class TestDoWithCleanups(CleanupsTestCase):
+
+    def trivial_func(self):
+        self.call_log.append('trivial_func')
+        return 'trivial result'
+
+    def test_runs_func(self):
+        """do_with_cleanups runs the function it is given, and returns the
+        result.
+        """
+        result = do_with_cleanups(self.trivial_func, [])
+        self.assertEqual('trivial result', result)
+
+    def test_runs_cleanups(self):
+        """Cleanup functions are run (in the given order)."""
+        cleanup_func_1 = lambda: self.call_log.append('cleanup 1')
+        cleanup_func_2 = lambda: self.call_log.append('cleanup 2')
+        do_with_cleanups(self.trivial_func, [cleanup_func_1, cleanup_func_2])
+        self.assertEqual(
+            ['trivial_func', 'cleanup 1', 'cleanup 2'], self.call_log)
+
+    def failing_func(self):
+        self.call_log.append('failing_func')
+        1/0
+
+    def test_func_error_propagates(self):
+        """Errors from the main function are propagated (after running
+        cleanups).
+        """
+        self.assertRaises(
+            ZeroDivisionError, do_with_cleanups, self.failing_func,
+            [self.no_op_cleanup])
+        self.assertEqual(['failing_func', 'no_op_cleanup'], self.call_log)
+
+    def test_func_error_trumps_cleanup_error(self):
+        """Errors from the main function a propagated even if a cleanup raises
+        an error.
+
+        The cleanup error is be logged.
+        """
+        self.assertRaises(
+            ZeroDivisionError, do_with_cleanups, self.failing_func,
+            [self.failing_cleanup])
+        self.assertLogContains('Cleanup failed:.*failing_cleanup goes boom')
+
+    def test_func_passes_and_error_from_cleanup(self):
+        """An error from a cleanup is propagated when the main function doesn't
+        raise an error.  Later cleanups are still executed.
+        """
+        exc = self.assertRaises(
+            Exception, do_with_cleanups, self.trivial_func,
+            [self.failing_cleanup, self.no_op_cleanup])
+        self.assertEqual('failing_cleanup goes boom!', exc.args[0])
+        self.assertEqual(
+            ['trivial_func', 'failing_cleanup', 'no_op_cleanup'],
+            self.call_log)
+
+    def test_multiple_cleanup_failures(self):
+        """When multiple cleanups fail (as tends to happen when something has
+        gone wrong), the first error is propagated, and subsequent errors are
+        logged.
+        """
+        cleanups = self.make_two_failing_cleanup_funcs()
+        self.assertRaises(ErrorA, do_with_cleanups, self.trivial_func,
+            cleanups)
+        self.assertLogContains('Cleanup failed:.*ErrorB')
+        log = self._get_log(keep_log_file=True)
+        self.assertFalse('ErrorA' in log)
+
+    def make_two_failing_cleanup_funcs(self):
+        def raise_a():
+            raise ErrorA('Error A')
+        def raise_b():
+            raise ErrorB('Error B')
+        return [raise_a, raise_b]
+
+    def test_multiple_cleanup_failures_debug_flag(self):
+        log = StringIO()
+        trace.push_log_file(log)
+        debug.debug_flags.add('cleanup')
+        cleanups = self.make_two_failing_cleanup_funcs()
+        self.assertRaises(ErrorA, do_with_cleanups, self.trivial_func, cleanups)
+        self.assertContainsRe(
+            log.getvalue(), "bzr: warning: Cleanup failed:.*Error B\n")
+        self.assertEqual(1, log.getvalue().count('bzr: warning:'),
+                log.getvalue())
+
+    def test_func_and_cleanup_errors_debug_flag(self):
+        log = StringIO()
+        trace.push_log_file(log)
+        debug.debug_flags.add('cleanup')
+        cleanups = self.make_two_failing_cleanup_funcs()
+        self.assertRaises(ZeroDivisionError, do_with_cleanups,
+            self.failing_func, cleanups)
+        self.assertContainsRe(
+            log.getvalue(), "bzr: warning: Cleanup failed:.*Error A\n")
+        self.assertContainsRe(
+            log.getvalue(), "bzr: warning: Cleanup failed:.*Error B\n")
+        self.assertEqual(2, log.getvalue().count('bzr: warning:'))
+
+    def test_func_may_mutate_cleanups(self):
+        """The main func may mutate the cleanups before it returns.
+        
+        This allows a function to gradually add cleanups as it acquires
+        resources, rather than planning all the cleanups up-front.
+        """
+        # XXX: this is cute, but an object with an 'add_cleanup' method may
+        # make a better API?
+        cleanups_list = []
+        def func_that_adds_cleanups():
+            self.call_log.append('func_that_adds_cleanups')
+            cleanups_list.append(self.no_op_cleanup)
+            return 'result'
+        result = do_with_cleanups(func_that_adds_cleanups, cleanups_list)
+        self.assertEqual('result', result)
+        self.assertEqual(
+            ['func_that_adds_cleanups', 'no_op_cleanup'], self.call_log)
+
+    def test_cleanup_error_debug_flag(self):
+        """The -Dcleanup debug flag causes cleanup errors to be reported to the
+        user.
+        """
+        log = StringIO()
+        trace.push_log_file(log)
+        debug.debug_flags.add('cleanup')
+        self.assertRaises(ZeroDivisionError, do_with_cleanups,
+            self.failing_func, [self.failing_cleanup])
+        self.assertContainsRe(
+            log.getvalue(),
+            "bzr: warning: Cleanup failed:.*failing_cleanup goes boom")
+        self.assertEqual(1, log.getvalue().count('bzr: warning:'))
+
+
+class ErrorA(Exception): pass
+class ErrorB(Exception): pass

